#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-3.0-or-later
#
# Copyright (C) 2018 IBM
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Malahal Naineni <malahal@us.ibm.com>

# Convert given Linux kernel NFS server exports into Ganesha exports.
#
# Typical usage:
# cat /etc/exports | knfs2ganesha-exports > /etc/ganesha/ganesha.conf
# cat /etc/exports | knfs2ganesha-exports --fsal gpfs > /etc/ganesha/ganesha.conf
#
# NOTE: default options with DASH/HYPHEN are not implemented!

from __future__ import print_function
import sys, os
import pyparsing as pp
import subprocess

# List of all export paths processed are kept in gan_paths dictionary
gan_paths = {}
export_id = 0 # increment before use

LPAREN = pp.Literal("(").suppress()
RPAREN = pp.Literal(")").suppress()

# export path can be quoted to include white space!
ppath = pp.QuotedString(quoteChar="'", escChar='\\',
                        unquoteResults=False,
                        convertWhitespaceEscapes=False)
ppath = pp.Or([ppath, pp.QuotedString(quoteChar='"', escChar='\\',
                                      unquoteResults=False,
                                      convertWhitespaceEscapes=False)])
ppath = pp.Or([ppath, pp.Word(pp.alphanums+"_-/.+")]).setResultsName("path")

phost = pp.Word(pp.alphanums+".@+*").setResultsName("host")
phost_opts = (LPAREN + pp.CharsNotIn(')') + RPAREN).setResultsName("host_opts")
phost_block = pp.Group(phost + phost_opts)
pblock = ppath + pp.OneOrMore(phost_block)

def process_exports(fsalname):
    for line in sys.stdin:
        line = line.strip()
        if not line or line.startswith("#"):
            continue

        p = pblock.ignore(pp.pythonStyleComment).parseString(line, parseAll=True)
        path = p["path"]
        host_blocks = p[1:]
        for host_block in host_blocks:
            host = host_block["host"]
            opts = host_block["host_opts"]
            if opts: # Not supposed as we don't support DASH based default options
                opts = opts[0]
            else:
                opts = ""
            #print(path, host, opts)
            pairs = process_opts(opts)
            create_client(path, host, pairs, fsalname)

# Takes a knfs options string and converts to Ganesha
# key,value pairs and returns a key,value list
def process_opts(opts):
    # Set to defaults
    access_value = "ro"
    squash_value = "root_squash"
    sec_value = "sys"

    pairs = []
    opts = opts.split(",")
    opts = [x.strip() for x in opts]
    for opt in opts:
        if opt in ["ro", "rw"]:
            access_value = opt
        elif opt in ["no_root_squash", "all_squash", "root_squash"]:
            squash_value = opt
        elif opt.startswith("sec"):
            auth = opt.split("=")[1]
            auth = auth.split(":")
            sec_value = ",".join(auth)
        elif opt in ["insecure", "sync", "no_subtree_check"]:
            pass
        elif opt == "async":
            sys.exit("async option is not supported in NFS-Ganesha, "
                     "more over, you shouldn't be using it if you "
                     "care about your data!\n"
                     "Please remove this option and re-run the program")
        elif opt == "subtree_check":
            sys.exit("subtree_check option is not supported in NFS-Ganesha.\n"
                     "Please remove this option and re-run the program")
        elif opt.startswith("fsid"):
            pass
        elif opt.startswith("anonuid"):
            uid = opt.split("=")[1].strip()
            pairs.append(("--Anonymous_uid", uid))
        elif opt.startswith("anongid"):
            gid = opt.split("=")[1].strip()
            pairs.append(("--Anonymous_gid", gid))
        else:
            sys.exit("Unknown option: %s, exiting" % opt)

    pairs.append(("--Access_Type", access_value))
    pairs.append(("--Squash", squash_value))
    pairs.append(("--SecType", sec_value))
    return pairs

def create_client(path, host, pairs, fsalname):
    global gan_paths
    global export_id

    # Create EXPORT{} block if needed
    if path not in gan_paths:
        gan_paths[path] = 1
        export_id += 1
        cmd = ["ganesha_conf", "set", "EXPORT", "Path", path]
        cmd += ["--Export_Id", str(export_id), "--Pseudo", path]
        subprocess.check_call(cmd)

    # Create EXPORT{FSAL{}} block
    cmd = ["ganesha_conf", "set", "EXPORT", "Path", path, "FSAL", "--name", fsalname]
    subprocess.check_call(cmd)

    # Create EXPORT{CLIENT{}} block with all key,value pairs
    cmd = ["ganesha_conf", "set", "EXPORT", "Path", path]
    cmd += ["CLIENT", "Clients", host]
    for key, value in pairs:
        cmd += [key, value]
    subprocess.check_call(cmd)

def usage(msg=""):
    msg += "\n%s [--fsal <fsal-name>]" % sys.argv[0]
    sys.exit(msg)

fsals = ["gpfs", "vfs", "lustre"]
def main():
    fsal = "vfs"
    argc = len(sys.argv)
    if argc == 1:
        pass
    elif argc == 2:
        if sys.argv[1].startswith("-h") or sys.argv[1].startswith("--help"):
            usage()
        else:
            msg = "Unknown parameter: %s" % sys.argv[1]
            usage(msg)
    elif argc == 3:
        if not sys.argv[1].startswith("--fsal"):
            msg = "Unknown parameter: %s" % sys.argv[2]
            usage(msg)
        fsal = sys.argv[2]
        if fsal not in fsals:
            msg = "Unknown fsal: %s" % fsal
            usage(msg)
    else:
        usage()

    import tempfile
    f = tempfile.NamedTemporaryFile()
    os.environ["CONFFILE"] = f.name
    process_exports(fsal)
    if export_id == 0:
        sys.exit("processed 0 exports!")
    else:
        # f.name is updated by subprocesses (ganesha_conf) and reading
        # "f" here gets nothing! Re-open does the trick!
        with open(f.name) as f2:
            for line in f2:
                sys.stdout.write(line)

if __name__ == '__main__':
    main()



#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-3.0-or-later
#
# Copyright (C) 2014 IBM
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Marc Eshel <eshel@us.ibm.com>
#
# This command receives statistics from Ganesha over DBus. The format
# for a command is:
#
# ganesha_stats <subcommand> <args>
#
# ganesha_stats help
#       To get detailed help
#
from __future__ import print_function
import sys
import Ganesha.glib_dbus_stats
import dbus

def print_usage_exit(return_code):
    message = (
"""

Usage:

Command displays global stats by default.

To display current status regarding stat counting use:
  {progname} status

To display stat counters use:
  {progname} [list_clients | deleg <ip address>
              inode | iov3 [export id] | iov4 [export id] | iov41 [export id] | iov42 [export id] |
              iomon [export id] | export | total [export id] | fast | pnfs [export id] |
              fsal <fsal name> | v3_full | v4_full | auth |
              client_io_ops <ip address> | export_details <export id> |
              client_all_ops <ip address>]

To display stat counters in json format use:
  {progname} json <command>

To reset stat counters use:
  {progname} reset

To enable/disable stat counters use:
  {progname} [enable | disable] [all | nfs | fsal | v3_full |
                                 v4_full | auth | client_all_ops]

"""
    )
    print(message.format(progname=sys.argv[0]))
    sys.exit(return_code)

output_json = False
if (len(sys.argv) < 2):
    command = 'global'
else:
    command = sys.argv[1]
    opts = sys.argv[2:]

if command == 'json':
    output_json = True
    if len(opts) >= 1:
        command = opts[0]
        opts = opts[1:]
    else:
        command = DEFAULT_COMMAND

json_not_available = ('fsal', 'reset', 'enable')
if output_json and command in json_not_available:
    sys.exit("{0} command is not available in json.".format(command))

# check arguments
commands = (
    'help', 'list_clients', 'deleg', 'global', 'inode', 'iov3', 'iov4',
    'iov41', 'iov42', 'iomon', 'export', 'total', 'fast', 'pnfs', 'fsal',
    'reset', 'enable', 'disable', 'status', 'v3_full', 'v4_full', 'auth',
    'client_io_ops', 'export_details', 'client_all_ops', 'json'
)

if command not in commands:
    print("\nError: Option '%s' is not correct." % command)
    print_usage_exit(1)
# requires an IP address
elif command in ('deleg', 'client_io_ops', 'client_all_ops'):
    if not len(opts) == 1:
        print("\nError: Option '%s' must be followed by an ip address." % command)
        print_usage_exit(1)
    command_arg = opts[0]
# requires an export id
elif command == 'export_details':
    if not len(opts) == 1:
        print("\nError: Option '%s' must be followed by an export id." % command)
        print_usage_exit(1)
    if opts[0].isdigit():
        command_arg = int(opts[0])
    else:
        print("\nError: Argument '%s' must be numeric." % opts[0])
        print_usage_exit(1)
# optionally accepts an export id
elif command in ('iov3', 'iov4', 'iov41', 'iov42', 'iomon', 'total', 'pnfs'):
    if (len(opts) == 0):
        command_arg = -1
    elif (len(opts) == 1) and opts[0].isdigit():
        command_arg = int(opts[0])
    else:
        print_usage_exit(1)
# requires fsal name
elif command in ('fsal'):
    if not len(opts) == 1:
        print("\nError: Option '%s' must be followed by fsal name." % command)
        print_usage_exit(1)
    command_arg = opts[0]
elif command in ('enable', 'disable'):
    if not len(opts) == 1:
        print("\nError: Option '%s' must be followed by all/nfs/fsal/v3_full/v4_full/auth/client_all_ops" %
            command)
        print_usage_exit(1)
    command_arg = opts[0]
    if command_arg not in ('all', 'nfs', 'fsal', 'v3_full', 'v4_full', 'auth', 'client_all_ops'):
        print("\nError: Option '%s' must be followed by all/nfs/fsal/v3_full/v4_full/auth/client_all_ops" %
            command)
        print_usage_exit(1)
elif command == "help":
    print_usage_exit(0)

# retrieve and print stats
try:
    exp_interface = Ganesha.glib_dbus_stats.RetrieveExportStats()
    cl_interface = Ganesha.glib_dbus_stats.RetrieveClientStats()

    result = None
    if command == "global":
        result = exp_interface.global_stats()
    elif command == "export":
        result = exp_interface.export_stats()
    elif command == "inode":
        result = exp_interface.inode_stats()
    elif command == "fast":
        result = exp_interface.fast_stats()
    elif command == "list_clients":
        result = cl_interface.list_clients()
    elif command == "deleg":
        result = cl_interface.deleg_stats(command_arg)
    elif command == "client_io_ops":
        result = cl_interface.client_io_ops_stats(command_arg)
    elif command == "client_all_ops":
        result = cl_interface.client_all_ops_stats(command_arg)
    elif command == "iov3":
        result = exp_interface.v3io_stats(command_arg)
    elif command == "iov4":
        result = exp_interface.v4io_stats(command_arg)
    elif command == "iov41":
        result = exp_interface.v41io_stats(command_arg)
    elif command == "iov42":
        result = exp_interface.v42io_stats(command_arg)
    elif command == "iomon":
        result = exp_interface.iomon_stats(command_arg)
    elif command == "total":
        result = exp_interface.total_stats(command_arg)
    elif command == "export_details":
        result = exp_interface.export_details_stats(command_arg)
    elif command == "pnfs":
        result = exp_interface.pnfs_stats(command_arg)
    elif command == "reset":
        result = exp_interface.reset_stats()
    elif command == "fsal":
        result = exp_interface.fsal_stats(command_arg)
    elif command == "v3_full":
        result = exp_interface.v3_full_stats()
    elif command == "v4_full":
        result = exp_interface.v4_full_stats()
    elif command == "auth":
        result = exp_interface.auth_stats()
    elif command == "enable":
        result = exp_interface.enable_stats(command_arg)
    elif command == "disable":
        result = exp_interface.disable_stats(command_arg)
    elif command == "status":
        result = exp_interface.status_stats()

    print(result.json()) if output_json else print(result)
except dbus.exceptions.DBusException:
    sys.exit("Error: Can't talk to ganesha service on d-bus. Looks like Ganesha is down")



#!/usr/bin/python3
# SPDX-License-Identifier: GPL-2.0-or-later
#
# ganesha_mgr_utils.py - commandline tool utils for managing nfs-ganesha.
#
# Copyright (C) 2014 IBM.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Allison Henderson <achender@vnet.linux.ibm.com>
#-*- coding: utf-8 -*-

import sys
import dbus
from collections import namedtuple

Client = namedtuple('Client',
                    ['ClientIP',
                     'HasNFSv3',
                     'HasMNT',
                     'HasNLM4',
                     'HasRQUOTA',
                     'HasNFSv40',
                     'HasNFSv41',
                     'HasNFSv42',
                     'Has9P',
                     'LastTime'])

class ClientMgr():

    def __init__(self, service, path, interface):
        self.dbus_service_name = service
        self.dbus_path = path
        self.dbus_interface = interface

        self.bus = dbus.SystemBus()
        try:
            self.dbusobj = self.bus.get_object(self.dbus_service_name,
                                               self.dbus_path)
        except:
            sys.exit("Error: Can't talk to ganesha service on d-bus." \
                     " Looks like Ganesha is down")

    def AddClient(self, ipaddr):
        add_client_method = self.dbusobj.get_dbus_method("AddClient",
                                                         self.dbus_interface)
        try:
            reply = add_client_method(ipaddr)
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def RemoveClient(self, ipaddr):
        remove_client_method = self.dbusobj.get_dbus_method("RemoveClient",
                                                            self.dbus_interface)
        try:
            reply = remove_client_method(ipaddr)
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def ShowClients(self):
        show_client_method = self.dbusobj.get_dbus_method("ShowClients",
                                                          self.dbus_interface)
        try:
            reply = show_client_method()
        except dbus.exceptions.DBusException as ex:
            return False, ex, []

        time = reply[0]
        client_array = reply[1]

        ts_ = (time[0], time[1])
        clients = []
        for client in client_array:
            cl_ = client
            lasttime = cl_[9]
            clt = Client(ClientIP=str(cl_[0]),
                         HasNFSv3=cl_[1],
                         HasMNT=cl_[2],
                         HasNLM4=cl_[3],
                         HasRQUOTA=cl_[4],
                         HasNFSv40=cl_[5],
                         HasNFSv41=cl_[6],
                         HasNFSv42=cl_[7],
                         Has9P=cl_[8],
                         LastTime=(lasttime[0],
                                   lasttime[1]))
            clients.append(clt)
        return True, "Done", [ts_, clients]



Export = namedtuple('Export',
                    ['ExportID',
                     'ExportPath',
                     'HasNFSv3',
                     'HasMNT',
                     'HasNLM4',
                     'HasRQUOTA',
                     'HasNFSv40',
                     'HasNFSv41',
                     'HasNFSv42',
                     'Has9P',
                     'LastTime'])

ExportClient = namedtuple('ExportClient',
                          ['Client_type',
                           'CIDR_version',
                           'CIDR_address',
                           'CIDR_mask',
                           'CIDR_proto',
                           'Anonymous_uid',
                           'Anonymous_gid',
                           'Expire_time_attr',
                           'Options',
                           'Set'])


class ExportMgr():
    '''
    org.ganesha.nfsd.exportmgr
    '''
    def __init__(self, service, path, interface):
        self.dbus_service_name = service
        self.dbus_path = path
        self.dbus_interface = interface
        self.bus = dbus.SystemBus()
        try:
            self.dbusobj = self.bus.get_object(self.dbus_service_name,
                                               self.dbus_path)
        except:
            sys.exit("Error: Can't talk to ganesha service on d-bus." \
                     " Looks like Ganesha is down")

    def AddExport(self, conf_path, exp_expr):
        add_export_method = self.dbusobj.get_dbus_method("AddExport",
                                                         self.dbus_interface)
        try:
            msg = add_export_method(conf_path, exp_expr)
        except dbus.exceptions.DBusException as ex:
            return False, ex

        return True, "Done: "+msg

    def UpdateExport(self, conf_path, exp_expr):
        update_export_method = self.dbusobj.get_dbus_method("UpdateExport",
                                                            self.dbus_interface)
        try:
            msg = update_export_method(conf_path, exp_expr)
        except dbus.exceptions.DBusException as ex:
            return False, ex

        return True, "Done: "+msg

    def RemoveExport(self, exp_id):
        rm_export_method = self.dbusobj.get_dbus_method("RemoveExport",
                                                        self.dbus_interface)
        try:
            rm_export_method(int(exp_id))
        except dbus.exceptions.DBusException as ex:
            return False, ex
        return True, "Done"

    def DisplayExport(self, exp_id):
        display_export_method = self.dbusobj.get_dbus_method("DisplayExport",
                                                             self.dbus_interface)
        try:
            id_, fullpath, pseudopath, tag, clients_array = \
                display_export_method(int(exp_id))
        except dbus.exceptions.DBusException as ex:
            return False, ex, []

        export_clients = []
        for client in clients_array:
            c_ = ExportClient(Client_type=client[0],
                              CIDR_version=client[1],
                              CIDR_address=client[2],
                              CIDR_mask=client[3],
                              CIDR_proto=client[4],
                              Anonymous_uid=client[5],
                              Anonymous_gid=client[6],
                              Expire_time_attr=client[7],
                              Options=client[8],
                              Set=client[9])

            export_clients.append(c_)

        return True, "Done", [id_, fullpath, pseudopath, tag, export_clients]

    def ShowExports(self):
        show_export_method = self.dbusobj.get_dbus_method("ShowExports",
                                                          self.dbus_interface)
        try:
            reply = show_export_method()
        except dbus.exceptions.DBusException as ex:
            return False, ex, []

        time = reply[0]
        export_array = reply[1]

        ts_ = (time[0], time[1])
        exports = []
        for export in export_array:
            ex = export
            lasttime = ex[10]
            exp = Export(ExportID=ex[0],
                         ExportPath=str(ex[1]),
                         HasNFSv3=ex[2],
                         HasMNT=ex[3],
                         HasNLM4=ex[4],
                         HasRQUOTA=ex[5],
                         HasNFSv40=ex[6],
                         HasNFSv41=ex[7],
                         HasNFSv42=ex[8],
                         Has9P=ex[9],
                         LastTime=(lasttime[0],
                                   lasttime[1]))
            exports.append(exp)
        return True, "Done", [ts_, exports]

class AdminInterface():
    '''
    org.ganesha.nfsd.admin interface
    '''
    def __init__(self, service, path, interface):
        self.dbus_service_name = service
        self.dbus_path = path
        self.dbus_interface = interface

        self.bus = dbus.SystemBus()
        try:
            self.dbusobj = self.bus.get_object(self.dbus_service_name,
                                               self.dbus_path)
        except:
            sys.exit("Error: Can't talk to ganesha service on d-bus." \
                     " Looks like Ganesha is down")

    def grace(self, ipaddr):
        grace_method = self.dbusobj.get_dbus_method("grace",
                                                    self.dbus_interface)
        try:
            reply = grace_method(ipaddr)
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def shutdown(self):
        shutdown_method = self.dbusobj.get_dbus_method("shutdown",
                                                       self.dbus_interface)
        try:
            reply = shutdown_method()
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def purge_netgroups(self):
        method = self.dbusobj.get_dbus_method("purge_netgroups",
                                              self.dbus_interface)
        try:
            reply = method()
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def purge_idmap(self):
        method = self.dbusobj.get_dbus_method("purge_idmapper_cache",
                                              self.dbus_interface)
        try:
            reply = method()
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def purge_gids(self):
        method = self.dbusobj.get_dbus_method("purge_gids",
                                              self.dbus_interface)
        try:
            reply = method()
        except dbus.exceptions.DBusException as ex:
            return False, ex

        status = reply[0]
        msg = reply[1]
        return status, msg

    def GetAll(self):
        method = self.dbusobj.get_dbus_method("GetAll",
                                              "org.freedesktop.DBus.Properties")
        try:
            dictionary = method(self.dbus_interface)
        except dbus.exceptions.DBusException as ex:
            return False, ex, {}

        prop_dict = {}
        for key in dictionary.keys():
            prop_dict[key] = dictionary[key]
        return True, "Done", prop_dict


IDMapper = namedtuple('IDMapper', ['Name', 'UID', 'HasGID', 'GID'])

FileSys = namedtuple('FileSys', ['Path', 'MajorDevId', 'MinorDevId'])

class CacheMgr():
    '''
    org.ganesha.nfsd.cachemgr
    '''
    def __init__(self, service, path, interface):
        self.dbus_service_name = service
        self.dbus_path = path
        self.dbus_interface = interface

        self.bus = dbus.SystemBus()
        try:
            self.dbusobj = self.bus.get_object(self.dbus_service_name,
                                               self.dbus_path)
        except:
            sys.exit("Error: Can't talk to ganesha service on d-bus." \
                     " Looks like Ganesha is down")

    def ShowFileSys(self):
        show_filesys_method = self.dbusobj.get_dbus_method("showfs",
                                                           self.dbus_interface)
        try:
            reply = show_filesys_method()
        except dbus.exceptions.DBusException as ex:
            return False, ex, []

        time = reply[0]
        fs_array = reply[1]

        ts_ = (time[0], time[1])

        fss = []
        for fs_ in fs_array:
            filesys1 = FileSys(Path=str(fs_[0]),
                               MajorDevId=fs_[1],
                               MinorDevId=fs_[2])
            fss.append(filesys1)
        return True, "Done", [ts_, fss]


    def ShowIdmapper(self):
        show_id_method = self.dbusobj.get_dbus_method("showidmapper",
                                                      self.dbus_interface)
        try:
            reply = show_id_method()
        except dbus.exceptions.DBusException as ex:
            return False, ex, []

        time = reply[0]
        id_array = reply[1]

        ts_ = (time[0], time[1])

        ids = []
        for entry in id_array:
            entry1 = IDMapper(Name=str(entry[0]),
                              UID=entry[1],
                              HasGID=entry[2],
                              GID=entry[3])
            ids.append(entry1)
        return True, "Done", [ts_, ids]


LOGGER_PROPS = 'org.ganesha.nfsd.log.component'

class LogManager():
    '''
    org.ganesha.nfsd.log.component
    '''

    def __init__(self, service, path, interface):
        self.dbus_service_name = service
        self.dbus_path = path
        self.dbus_interface = interface

        self.bus = dbus.SystemBus()
        try:
            self.dbusobj = self.bus.get_object(self.dbus_service_name,
                                               self.dbus_path)
        except:
            sys.exit("Error: Can't talk to ganesha service on d-bus. " \
                     " Looks like Ganesha is down")

    def GetAll(self):
        getall_method = self.dbusobj.get_dbus_method("GetAll",
                                                     self.dbus_interface)
        try:
            dictionary = getall_method(LOGGER_PROPS)
        except dbus.exceptions.DBusException as ex:
            return False, ex, {}

        prop_dict = {}
        for key in dictionary.keys():
            prop_dict[key] = dictionary[key]
        return True, "Done", prop_dict

    def Get(self, prop):
        get_method = self.dbusobj.get_dbus_method("Get",
                                                  self.dbus_interface)
        try:
            level = get_method(LOGGER_PROPS, prop)
        except dbus.exceptions.DBusException as ex:
            return False, ex, 0

        return True, "Done", level

    def Set(self, prop, setval):
        set_method = self.dbusobj.get_dbus_method("Set",
                                                  self.dbus_interface)
        try:
            set_method(LOGGER_PROPS, prop, setval)
        except dbus.exceptions.DBusException as ex:
            return False, ex

        return True, "Done"



# SPDX-License-Identifier: LGPL-3.0-or-later
#
# admin.py - AdminInterface DBUS object class.
#
# Copyright (C) 2014 Panasas Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Jim Lieb <jlieb@panasas.com>
#-*- coding: utf-8 -*-

from PyQt5 import QtDBus

class AdminInterface(QtDBus.QDBusAbstractInterface):
    '''
    org.ganesha.nfsd.admin interface
    '''
    def __init__(self, service, path, connection, show_status, parent=None):
        super(AdminInterface, self).__init__(service,
                                             path,
                                             'org.ganesha.nfsd.admin',
                                             connection,
                                             parent)
        self.show_status = show_status

    def grace(self, ipaddr):
        _async = self.asyncCall("grace", ipaddr)
        status = QtDBus.QDBusPendingCallWatcher(_async, self)
        status.finished.connect(self.admin_done)

    def reload(self):
        _async = self.asyncCall("reload")
        status = QtDBus.QDBusPendingCallWatcher(_async, self)
        status.finished.connect(self.admin_done)

    def shutdown(self):
        _async = self.asyncCall("shutdown")
        status = QtDBus.QDBusPendingCallWatcher(_async, self)
        status.finished.connect(self.admin_done)

    # catch the reply and forward it to the UI
    def admin_done(self, call):
        reply = QtDBus.QDBusPendingReply(call)
        if reply.isError():
            self.show_status.emit(False,
                                  "DBUS error:" + str(reply.error().message()))
        else:
            status = reply.argumentAt(0).toPyObject()
            msg = reply.argumentAt(1).toPyObject()
            self.show_status.emit(status, msg)




#!/usr/bin/python3
# SPDX-License-Identifier: GPL-3.0-or-later
#
# manage_exports.py - commandline tool for managing exports in nfs-ganesha.
#
# Copyright (C) 2014 Panasas Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Jim Lieb <jlieb@panasas.com>
#-*- coding: utf-8 -*-
from __future__ import print_function
import sys
import time
from PyQt5 import QtCore, QtDBus
from PyQt5.QtGui import QApplication
from dbus.mainloop.qt import DBusQtMainLoop
from Ganesha.export_mgr import ExportMgr

SERVICE = 'org.ganesha.nfsd'

class ShowExports(QtCore.QObject):

    show_status = QtCore.pyqtSignal(bool, str)

    def __init__(self, sysbus, parent=None):
        super(ShowExports, self).__init__()
        self.exportmgr = ExportMgr(SERVICE,
                                   '/org/ganesha/nfsd/ExportMgr',
                                   sysbus, self.show_status)
        self.show_status.connect(self.status_message)
        self.exportmgr.show_exports.connect(self.proc_exports)
        self.exportmgr.display_export.connect(self.proc_export)

    def showexports(self):
        self.exportmgr.ShowExports()
        print("Show exports")

    def addexport(self, conf_path, exp_expr):
        self.exportmgr.AddExport(conf_path, exp_expr)
        print("Add Export in %s" % conf_path)

    def updateexport(self, conf_path, exp_expr):
        self.exportmgr.UpdateExport(conf_path, exp_expr)
        print("Update Export in %s" % conf_path)

    def removeexport(self, exp_id):
        self.exportmgr.RemoveExport(exp_id)
        print("Remove Export with id %d" % int(exp_id))

    def displayexport(self, exp_id):
        self.exportmgr.DisplayExport(exp_id)
        print("Display export with id %d" % int(exp_id))

    def proc_export(self, _id, path, pseudo, tag):
        print("export %d: path = %s, pseudo = %s, tag = %s" % (_id, path, pseudo, tag))
        sys.exit()

    def proc_exports(self, _ts, exports):
        print("Timestamp: ", time.ctime(_ts[0]), _ts[1], " nsecs")
        if len(exports) == 0:
            print("No exports")
        else:
            print("Exports:")
            print("  Id, path,    nfsv3, mnt, nlm4, rquota,nfsv40, nfsv41, 9p, last")
            for export in exports:
                print(" %d,  %s,  %s,  %s,  %s,  %s,  %s,  %s,  %s,  %s %d nsecs" %
                      (export.ExportID,
                       export.ExportPath,
                       export.HasNFSv3,
                       export.HasMNT,
                       export.HasNLM4,
                       export.HasRQUOTA,
                       export.HasNFSv40,
                       export.HasNFSv41,
                       export.Has9P,
                       time.ctime(export.LastTime[0]), export.LastTime[1]))
        sys.exit()

    def status_message(self, status, errormsg):
        print("Error: status = %s, %s" % (str(status), errormsg))
        sys.exit()

# Main
if __name__ == '__main__':
    app = QApplication(sys.argv)
    loop = DBusQtMainLoop(set_as_default=True)
    sysbus = QtDBus.QDBusConnection.systemBus()
    exportmgr = ShowExports(sysbus)
    if sys.argv[1] == "add":
        exportmgr.addexport(sys.argv[2], sys.argv[3])
    elif sys.argv[1] == "update":
        exportmgr.updateexport(sys.argv[2], sys.argv[3])
    elif sys.argv[1] == "remove":
        exportmgr.removeexport(sys.argv[2])
    elif sys.argv[1] == "display":
        exportmgr.displayexport(sys.argv[2])
    elif sys.argv[1] == "show":
        exportmgr.showexports()
    else:
        print("Unknown/missing command")
        sys.exit()
    app.exec_()
